/* keypad.h */
#ifndef KEYPAD_H
#define KEYPAD_H

#include "\ulib\ulib.h"

#define ENABLE_DIAGNOSTICS   0

#define RESET_AKC  0
#define RESET_AMC  1

#define ON                   0
#define OFF                  !ON

#define UNLIT                1
#define LIT                  !UNLIT

#define SET                  1
#define CLEAR                !SET

#define ENABLED              0
#define DISABLED             !ENABLED

#define SEND_BUFFER_SIZE     16
#define RCV_BUFFER_SIZE      16
#define HEARTBEAT_PULSE      2000
#define BLIP_DURATION        4
#define BEEP_DURATION        80
#define ALERT_BEEP_DURATION  30
#define CODE_VALID_BUZZ_DURATION 25

#define ALREADY_OPEN_BEEPS   12
#define CART_ACTIVE_BEEPS    3

#define STAR_KEY            10
#define POUND_KEY           11
#define NUMBER_OF_KEYS      12

#define DEBOUNCE_DELAY     30       // in milliseconds

/* pin definitions */
#define HEARTBEAT_LED_PIN  RA4
#define TEST_SWITCH_PIN    RB7
#define BUZZER_PIN         RA5
#define SERVICE_LED_PIN    RC0
#define SLEEP_LED_PIN      RC1
#define READY_LED_PIN      RC2
#define LOW_BATT_LED_PIN	RC5
#define CHARGING_LED_PIN   RC6
#define OPEN_LED_PIN       RC7

/* LED identifiers */
#define SERVICE_LED      0x01
#define	SLEEP_LED       0x02
#define	READY_LED       0x03
#define	OPEN_LED        0x04
#define CHARGING_LED    0x05
#define	LOW_BATT_LED    0x06

#define BUZZER          0x0F

/* AKC->AMC request codes */
#define	OPEN_DRAWER     0x10
#define	CODE_VERIFY     0x20
#define	DIAGNOSTIC      0x30

/* AMC->AKC command codes */
#define  NO_COMMAND                 0xAA
#define  NO_DATA                    0xAB
#define  BUZZER_ON 	               0x4F
#define  BUZZER_OFF 	               0x5F
#define  OUTPUT_ON	               0x40
#define  OUTPUT_OFF		            0x50
#define  BEEP			               0x61
#define  BLIP 			               0x62
#define  CART_ACTIVE_ALERT          0x63
#define  DRAWER_ALREADY_OPEN_ALERT  0x64
#define  CODE_IS_VALID              0x65
#define  CODE_NOT_VALID             0x66

/* execution states */
#define  UNKNOWN_STATE  0x00
#define  IDLE_STATE     0x01
#define  OUTPUT_STATE   0x02
#define  INPUT_STATE    0x04
#define  WAITING_STATE  0x08
#define  SENDING_CODE_STATE   0x10
#define  SENDING_POP_STATE    0x20
#define  SENDING_DUMMY_STATE  0x40

#define  AKC_CMD_MASK           0xF0
#define  AKC_OUTPUT_MASK        0x0F

/* I2C flag comparisons */
#define I2C_MASTER_READ_COMMAND   1
#define I2C_MASTER_WRITE_COMMAND  0
#define I2C_BUFFER_FULL		  1
#define I2C_BUFFER_EMPTY	  0
#define I2C_DATA_RECEIVED	  1
#define I2C_ADDRESS_RECEIVED	  0

struct
command_buffer
{
  int count;
  char buffer[SEND_BUFFER_SIZE];
};

/* macros */

#define mAKC_ForceState(__a__)  theState = __a__
#define mAKC_CheckState(__a__)  theState&__a__
#define mAKC_ToggleState(__a__)  theState^=__a__
#define mAKC_SetState(__a__)    theState|=__a__
#define mAKC_ClearState(__a__)  {if(theState&__a__) theState^=__a__;}
#define mAKC_ChangeState(__a__,__b__) {mAKC_ClearState(__a__);mAKC_SetState(__b__);}

#if ENABLE_DIAGNOSTICS

//#define mAKC_DoDiagnostic(__a__) {\
//                                  if (TEST_SWITCH_PIN==ON) __a__ = LIT;\
//                                  else __a__ = UNLIT;\
//                                 }
#else
#define mAKC_DoDiagnostic(__a__)
#endif


#define mAKC_SignalAMC()        RB6 = 0
#define mAKC_SignalAMCReset()   RB6 = 1
#define mAKC_RebootAMC()        {RB5 = 0;TRISB5 = 0;}
#define mAKC_AMCIsSignalled     !RB6
#define mAKC_Settle()            asm("nop")
#define mAKC_ReleaseI2C()        CKP = 1

/* prototypes */
void AKC_Reboot();

BYTE AKC_RawReadKeypad(BYTE* key_value,BYTE* key_press);
int  AKC_Keysequal(void);
int  AKC_ReadKeypad(void);
int  AKC_ReadKeyNum(void);
char AKC_ReadKey(void);
void AKC_AccumKeypad(char c);
void AKC_ProcessKeypad(void);
void AKC_Beep(void);
void AKC_Blip(void);
void AKC_CartActiveAlert(void);
void AKC_DrawerAlreadyOpenAlert(void);
void AKC_CodeIsValidBuzz(void);
void AKC_PopDrawer(char drawer);
void AKC_Wait(unsigned int duration);

#endif  // end keypad.h


